/* Copyright © 2013-2020 Graphia Technologies Ltd.
 *
 * This file is part of Graphia.
 *
 * Graphia is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Graphia is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Graphia.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef IGRAPHMODEL_H
#define IGRAPHMODEL_H

#include "shared/graph/elementid.h"
#include "shared/graph/elementtype.h"

#include <vector>
#include <QString>

class IGraph;
class IMutableGraph;
class IAttribute;
struct IElementVisual;

class IGraphModel
{
public:
    virtual ~IGraphModel() = default;

protected:
    virtual IMutableGraph& mutableGraphImpl() = 0;
    virtual const IMutableGraph& mutableGraphImpl() const = 0;
    virtual const IGraph& graphImpl() const = 0;

    virtual const IElementVisual& nodeVisualImpl(NodeId nodeId) const = 0;
    virtual const IElementVisual& edgeVisualImpl(EdgeId edgeId) const = 0;

public:
    IMutableGraph& mutableGraph() { return mutableGraphImpl(); }
    const IMutableGraph& mutableGraph() const { return mutableGraphImpl(); }
    const IGraph& graph() const { return graphImpl(); }

    const IElementVisual& nodeVisual(NodeId nodeId) const { return nodeVisualImpl(nodeId); }
    const IElementVisual& edgeVisual(EdgeId edgeId) const { return edgeVisualImpl(edgeId); }

    virtual QString nodeName(NodeId nodeId) const = 0;
    virtual void setNodeName(NodeId nodeId, const QString& name) = 0;

    virtual IAttribute& createAttribute(QString name) = 0;
    virtual const IAttribute* attributeByName(const QString& name) const = 0;
    virtual bool attributeExists(const QString& name) const = 0;
    virtual std::vector<QString> attributeNames(ElementType elementType = ElementType::All) const = 0;
};

#endif // IGRAPHMODEL_H
